#include "common.h"
#include "simulator.h"
#include <stdio.h>

int num_blocks(u_char* packet, int len) /*{{{*/
{
	int skip_len = radiotap_len(packet) + sizeof(struct wlan_hdr) + LLCSIZE + IPSIZE + UDPSIZE;
	len -= skip_len + 0x30; /* 30 is for the iperf bits that we have no idea about */
	return ceil((double)len / (double)BLOCKSIZE);
}
/*}}}*/

int num_good_contig_blocks(u_char* packet, int len) /*{{{*/
{
	int num_goodblocks = 0;
	int i;

	int skip_len = radiotap_len(packet) + sizeof(struct wlan_hdr) + LLCSIZE + IPSIZE + UDPSIZE + 4; // 4 is for the 80211 checksum

	packet += skip_len;
	len -= skip_len + 4; // 4 is for the 80211 checksum

	for (i = 0; i < len; i++)
	{
		if (i == 2 && i == 3)
				continue;

		if (packet[i] != 0)
			return num_goodblocks;

		/* If at the end of the block, and something was bad, increment the counter */
		if (i % BLOCKSIZE == 0)
		{
			num_goodblocks++;	
		}
	}
	return num_goodblocks;
} /*}}}*/

int num_good_contig_blocks_iperf(u_char* packet, int len) /*{{{*/
{
	int num_goodblocks = 0;
	int i;

	int base = 0x30;

	int skip_len = radiotap_len(packet) + sizeof(struct wlan_hdr) +
		LLCSIZE + IPSIZE + UDPSIZE;  
	packet += skip_len;
	len -= skip_len + 4; // 4 is for the 80211 checksum

	printf("len: %d\n", len);
	for (i = 36; i < len; i++) /* Skip over 40 bytes of the packet due to iperf header */
	{
		// Bad block, return
		if (packet[i] != base+(i%10)) 
		{
			//printf("bad at %d: %02x != %02x\n", i, packet[i], base + (i % 10));
			return num_goodblocks;
		}

		/* If at the end of the block, and something was bad, increment the counter */
		if (i % BLOCKSIZE == 0)
		{
			num_goodblocks++;	
		}
	}
	// The last block that is outside of the block size
	if (len % BLOCKSIZE != 0)
	{
		num_goodblocks++;	
	}

	return num_goodblocks;
} /*}}}*/

void print_good_blocks_iperf(u_char* packet, int len, float rate, uint64_t start_time) /*{{{*/
{
	int num_goodblocks = 0;
	int bad_block = 0;
	int i;

	int base = 0x30;

	int skip_len = radiotap_len(packet) + sizeof(struct wlan_hdr) +
		LLCSIZE + IPSIZE + UDPSIZE;
	packet += skip_len;
	len -= skip_len + 4; // 4 is for the 80211 checksum

	int prev_bad = -1;
	uint64_t num_blocks = 0;
	uint64_t curr_start_time = start_time;

	printf("%lld PACKET 0 %lld GOOD_BLOCK %f 0\n",
						curr_start_time,
						packet_transmission_time_us(skip_len+36, rate),
						rate);
	printf("%lld SIM-PACKET 0 %lld GOOD_BLOCK %f 0\n",
						curr_start_time,
						packet_transmission_time_us(skip_len+36, rate),
						rate);
	curr_start_time += packet_transmission_time_us(skip_len+36, rate);
	/* todo: find a way not to assume the first 36 bytes are fine */
	for (i = 36; i < len; i++) /* Skip over 36 bytes of the packet due to iperf header */
	{
		/* Flag block as bad */
		if (packet[i] != base+(i%10)) {
			bad_block = 1;
		}

		/* If at the end of the block, and nothing bad, increment the counter */
		if (i % BLOCKSIZE == 0)
		{
			if (prev_bad != -1 && prev_bad != bad_block)
			{
				if (!prev_bad)
				{
					printf("%lld PACKET 0 %lld GOOD_BLOCK %f 0\n",
						curr_start_time,
						packet_transmission_time_us(BLOCKSIZE, rate)*num_blocks,
						rate);
					printf("%lld SIM-PACKET 0 %lld GOOD_BLOCK %f 0\n",
						curr_start_time,
						packet_transmission_time_us(BLOCKSIZE, rate)*num_blocks,
						rate);
				}
				else
				{
					printf("%lld PACKET 0 %lld BAD_BLOCK %f 0\n",
						curr_start_time,
						packet_transmission_time_us(BLOCKSIZE, rate)*num_blocks,
						rate);
					printf("%lld SIM-PACKET 0 %lld BAD_BLOCK %f 0\n",
						curr_start_time,
						packet_transmission_time_us(BLOCKSIZE, rate)*num_blocks,
						rate);
				}
				curr_start_time += packet_transmission_time_us(BLOCKSIZE, rate)*num_blocks;
				num_blocks = 0;
			}
			prev_bad = bad_block;
			bad_block = 0;
			num_blocks++;
		}
	}
	// The last block that is outside of the block size
	if (len % BLOCKSIZE != 0)
	{
		num_blocks++;
		if (!prev_bad)
		{
				printf("%lld PACKET 0 %lld GOOD_BLOCK %f 0\n",
					curr_start_time,
					packet_transmission_time_us(BLOCKSIZE, rate)*num_blocks,
					rate);
				printf("%lld SIM-PACKET 0 %lld GOOD_BLOCK %f 0\n",
					curr_start_time,
					packet_transmission_time_us(BLOCKSIZE, rate)*num_blocks,
					rate);
		}
		else
		{
				printf("%lld PACKET 0 %lld BAD_BLOCK %f 0\n",
					curr_start_time,
					packet_transmission_time_us(BLOCKSIZE, rate)*num_blocks,
					rate);
				printf("%lld SIM-PACKET 0 %lld BAD_BLOCK %f 0\n",
					curr_start_time,
					packet_transmission_time_us(BLOCKSIZE, rate)*num_blocks,
					rate);
		}
	}
} /*}}}*/

int num_good_blocks_iperf_in_first_n_blocks(u_char* packet, int len, int num_blocks_simulated) /*{{{*/
{
	int num_goodblocks = 0;
	int bad_block = 0;
	int i;

	int base = 0x30;

	int skip_len = radiotap_len(packet) + sizeof(struct wlan_hdr) +
		LLCSIZE + IPSIZE + UDPSIZE;
	packet += skip_len;
	len -= skip_len + 4; // 4 is for the 80211 checksum

	printf("ngbiifnb: len %d, nbs %d\n", len, num_blocks_simulated);

	/* todo: find a way not to assume the first 36 bytes are fine */
	for (i = 36; i < len && i < num_blocks_simulated * BLOCKSIZE; i++) /* Skip over 36 bytes of the packet due to iperf header */
	{
		/* Flag block as bad */
		if (packet[i] != base+(i%10)) {
			//printf("bad at %d: %02x != %02x\n", i, packet[i], base + (i % 10));
			bad_block = 1;
		}

		/* If at the end of the block, and nothing bad, increment the counter */
		if (i % BLOCKSIZE == 0)
		{
			if (!bad_block)
				num_goodblocks++;	
			bad_block = 0;
		}
	}
	// The last block that is outside of the block size
	if (len % BLOCKSIZE != 0)
	{
	if (!bad_block)
		num_goodblocks++;	
	}

	printf("There were %d good blocks IPERF\n", num_goodblocks);
	return num_goodblocks;
} /*}}}*/

int num_good_blocks_iperf(u_char* packet, int len) /*{{{*/
{
	return num_good_blocks_iperf_in_first_n_blocks(packet, len, num_blocks(packet, len));
}

